<?php
/**
 * @package     CSVI
 * @subpackage  J2Store
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - [year] RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace j2store\com_j2store\model\import;

defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\Utilities\ArrayHelper;

/**
 * Product import.
 *
 * @package     CSVI
 * @subpackage  J2Store
 * @since       7.3.0
 */
class Product extends \RantaiImportEngine
{
	/**
	 * CSVI fields
	 *
	 * @var    \CsviHelperImportFields
	 * @since  7.3.0
	 */
	protected $fields;

	/**
	 * The addon helper
	 *
	 * @var    \Com_J2StoreHelperCom_J2Store
	 * @since  7.3.0
	 */
	protected $helper;

	/**
	 * Product table
	 *
	 * @var    \J2StoreTableProduct
	 * @since  7.3.0
	 */
	private $productTable;

	/**
	 * Joomla content table
	 *
	 * @var    \J2StoreTableContent
	 * @since  7.3.0
	 */
	private $contentTable;

	/**
	 * Variant table
	 *
	 * @var    \J2StoreTableContent
	 * @since  7.3.0
	 */
	private $variantTable;

	/**
	 * Product image table
	 *
	 * @var    \J2StoreTableProductimage
	 * @since  7.3.0
	 */
	private $imageTable;

	/**
	 * The image helper
	 *
	 * @var    \CsviHelperImage
	 * @since  7.3.0
	 */
	protected $imageHelper;

	/**
	 * The custom fields that can be used as available field.
	 *
	 * @var    array
	 * @since  7.15.0
	 */
	private $customFields = array();

	/**
	 * The categories import model
	 *
	 * @since  7.15.0
	 */
	private $categoryImportModel;

	/**
	 * Run this before we start.
	 *
	 * @return  void.
	 *
	 * @throws  \Exception
	 *
	 * @since   7.15.0
	 */
	public function onBeforeStart()
	{
		// Load the custom fields
		$this->loadCustomFields();
		$this->initialiseCategoriesImport();
	}

	/**
	 * Start the menu import process.
	 *
	 * @return  bool  True on success | false on failure.
	 *
	 * @since   7.3.0
	 *
	 * @throws  \RuntimeException
	 * @throws  \InvalidArgumentException
	 * @throws  \UnexpectedValueException
	 */
	public function getStart()
	{
		// Process data
		foreach ($this->fields->getData() as $fields)
		{
			foreach ($fields as $name => $details)
			{
				$value = $details->value;

				switch ($name)
				{
					case 'manufacturer_name':
						$this->setState('manufacturer_id', $this->helper->getManufacturerId($value));
						break;
					case 'vendor_user_email':
						$this->setState('vendor_id', $this->helper->getVendorId($value));
						break;
					case 'category_path':

						$categoryPath = $value;

						// Check if there are multiple categories and take only the first one
						if (strpos($value, '|') !== false)
						{
							$splitCategory = explode('|', $value);
							$categoryPath  = array_shift($splitCategory);
						}

						$categoryId = $this->helper->getCategoryId($this->categoryImportModel->createPath($categoryPath));

						if (!$categoryId)
						{
							$categoryId = $this->runCategoryImport($value);

							if (!$categoryId)
							{
								$categoryId = 2;
								$this->log->add('No category id found so storing in default category');
							}
						}

						$this->setState('catid', $categoryId);
						break;
					case 'taxprofile_name':
						$this->setState('taxprofile_id', $this->helper->getTaxProfileId($value));
						break;
					case 'enabled':
					case 'is_master':
					case 'manage_stock':
					case 'quantity_restriction':
					case 'shipping':
					case 'visibility':
						switch (strtolower($value))
						{
							case 'n':
							case 'no':
							case '0':
								$value = 0;
								break;
							default:
								$value = 1;
								break;
						}

						$this->setState($name, $value);
						break;
					default:
						$this->setState($name, $value);
						break;
				}
			}
		}

		$field = $selectField = 'j2store_product_id';
		$table = '#__j2store_products';

		if ($this->template->get('update_based_on', 'sku'))
		{
			$field = $this->template->get('update_based_on', 'sku');

			$productFields = $this->productTable->getFields();
			$variantFields = $this->variantTable->getFields();

			if ($this->getState($field, false))
			{
				if (isset($productFields[$field]))
				{
					$this->productTable->setKeyName($field);
					$this->variantTable->setKeyName('product_id');
					$table = '#__j2store_products';
					$selectField = "j2store_product_id";
				}
				elseif (isset($variantFields[$field]))
				{
					$this->productTable->setKeyName('j2store_product_id');
					$this->variantTable->setKeyName($field);
					$table = '#__j2store_variants';
					$selectField = "product_id";
				}
			}
		}

		// If no category is found set to default
		if (!$this->getState('catid', false))
		{
			$this->setState('catid', 2);
		}

		// If no alias set use the title field
		$requiredField = $this->getState($field, false);

		if (!$requiredField)
		{
			$this->loaded = false;
			$this->log->addStats('skipped', Text::sprintf('COM_CSVI_NO_REQUIRED_FIELD_FOUND', $field));
		}
		else
		{
			$this->recordIdentity = $this->getState($field, '');
			$this->loaded = true;
			$title  = $this->getState('title', false);
			$alias  = $this->getState('alias', false);

			if (!$this->getState('j2store_product_id', false))
			{
				// Check if we have a parent for a variable product type
				if ($this->getState('product_type') === 'variable')
				{
					if ($title || $alias)
					{
						// This may not have the SKU set, need to find the ID via the required field
						$productId = $this->helper->getProductIdByContent($alias, $title);

						$this->setState('j2store_product_id', $productId);
					}
					else
					{
						// We cannot find the parent, this is where it ends
						$this->loaded = false;

						return true;
					}
				}
				else
				{
					$this->setState('j2store_product_id', $this->helper->getProductId($field, $this->getState($field, ''), $table, $selectField));
				}
			}

			if (!$title && !$this->getState('j2store_product_id', false))
			{
				$this->setState('title', $this->getState($field, false));
			}

			if (!$alias && $this->getState('title', false))
			{
				$alias = \JFilterOutput::stringURLSafe($this->getState('title', false));
				$this->setState('alias', $alias);
			}

			// If no title set then use alias as title
			if ($alias && !$this->getState('title', false))
			{
				$this->setState('title', $alias);
			}

			$alias = $this->getState('alias', false);

			if ($alias && $this->getState('catid', false))
			{
				$this->setState('id', $this->helper->getContentId($this->getState('alias', false), $this->getState('catid', false)));
			}

			if (!$this->template->get('products_existing_content', false) && $alias && !$this->getState('j2store_product_id', false))
			{
				if (!$this->contentTable->checkAlias($alias))
				{
					$this->log->addStats('skipped', Text::sprintf('COM_CSVI_PRODUCT_ALIAS_EXISTS', $alias));

					$this->loaded = false;
				}
			}

			if ($this->productTable->load($this->getState($field, false)) || $this->variantTable->load($this->getState($field, false)))
			{
				if (!$this->template->get('overwrite_existing_data'))
				{
					$this->log->addStats('skipped', Text::sprintf('COM_CSVI_DATA_EXISTS_PRODUCT_SKU', $this->getState($field, '')));
					$this->loaded = false;
				}
				else
				{
					$this->setState('j2store_variant_id', $this->variantTable->get('j2store_variant_id', 0));
					$this->setState('j2store_product_id', $this->getState('j2store_product_id', 0));
					$this->setState('id', $this->getState('id', 0));
				}
			}
			else
			{
				$productId = $this->getState('j2store_product_id', false);

				if (!$this->getState($field, false) && empty($productId))
				{
					$this->log->addStats('incorrect', 'COM_CSVI_DEBUG_NO_SKU');
					$this->log->add(Text::_('COM_CSVI_DEBUG_NO_SKU'));

					$this->loaded = false;
				}
				else
				{
					// Product is not found so we need to reset to the primary key field
					$this->productTable->setKeyName('j2store_product_id');
					$this->variantTable->setKeyName('j2store_variant_id');
					$this->contentTable->setKeyName('id');
					$this->log->add(Text::sprintf('COM_CSVI_DEBUG_PROCESS_SKU', $this->recordIdentity), false);
				}
			}
		}

		return true;
	}

	/**
	 * Process a record.
	 *
	 * @return  bool  Returns true if all is OK | Returns false if no path or menu ID can be found.
	 *
	 * @since   7.3.0
	 *
	 * @throws  \RuntimeException
	 * @throws  \InvalidArgumentException
	 * @throws  \UnexpectedValueException
	 */
	public function getProcessRecord()
	{
		if (!$this->loaded)
		{
			return false;
		}

		if (!$this->getState('j2store_product_id', false) && $this->template->get('ignore_non_exist'))
		{
			$this->log->addStats('skipped', Text::sprintf('COM_CSVI_DATA_EXISTS_IGNORE_NEW', $this->recordIdentity));
		}
		else
		{
			$this->productTable->load($this->getState('j2store_product_id', false));

			if (!$this->getState('alias', false) && $this->getState('title', false))
			{
				$alias = \JFilterOutput::stringURLSafe($this->getState('title', false));
				$this->setState('alias', $alias);
			}

			if (!$this->getState('product_source_id', false))
			{
				$this->setState('product_source_id', $this->helper->getProductSourceId($this->getState('j2store_product_id', false)));
			}

			// First step is to save the product as a Joomla article
			$this->storeJoomlaContent();
			$this->log->add('Store product as Joomla content', false);

			if (!$this->getState('product_source', false))
			{
				$this->productTable->product_source = 'com_content';
			}

			if (!$this->getState('product_type', false))
			{
				$this->productTable->product_type = 'simple';
			}

			// Set the params for a product
			$this->setParams();

			if (!$this->getState('j2store_product_id', false))
			{
				if (!$this->getState('created_by'))
				{
					$this->productTable->created_by = $this->userId;
				}

				if (!$this->getState('created_on'))
				{
					$this->productTable->created_on = $this->date->toSql();
				}

				if (!$this->getState('taxprofile_id'))
				{
					$this->productTable->taxprofile_id = 0;
				}

				if (!$this->getState('vendor_id'))
				{
					$this->productTable->vendor_id = 0;
				}

				if (!$this->getState('manufacturer_id'))
				{
					$this->productTable->manufacturer_id = 0;
				}

				if (!$this->getState('params'))
				{
					$params = '{"product_css_class":""}';

					if ($this->getState('product_type', false) === 'downloadable')
					{
						$params = '{"product_css_class":"","download_limit":"","download_expiry":""}';
					}

					$this->productTable->params = $params;
					$this->setState('params', $params);
				}
			}
			else
			{
				if (!$this->getState('modified_on', false))
				{
					$this->productTable->modified_on = $this->date->toSql();
				}

				if (!$this->getState('modified_by'))
				{
					$this->productTable->modified_by = $this->userId;
				}
			}

			if ($this->getState('up_sells', false))
			{
				$newUpSellProductIds = array();
				$relatedUpSellProducts = explode('|', $this->getState('up_sells', false));

				foreach ($relatedUpSellProducts as $product)
				{
					$productId = $this->helper->getProductId('sku', $product);
					$newUpSellProductIds[] = $productId;
				}

				$this->setState('up_sells', implode(',', $newUpSellProductIds));
				$this->log->add('Product up_sells added');
			}

			if ($this->getState('cross_sells', false))
			{
				$newCrossSellProductIds = array();
				$relatedCrossSellProducts = explode('|', $this->getState('cross_sells', false));

				foreach ($relatedCrossSellProducts as $crossSellProduct)
				{
					$crossProductId = $this->helper->getProductId('sku', $crossSellProduct);
					$newCrossSellProductIds[] = $crossProductId;
				}

				$this->setState('cross_sells', implode(',', $newCrossSellProductIds));
				$this->log->add('Product cross_sells added');
			}

			// If the product is variable type and has options to generate variants
			if ($this->getState('product_type') === 'variable')
			{
				$this->productTable->has_options = 1;
			}

			// Set the default value
			$this->productTable->enabled = 1;

			if (strtoupper($this->getState('product_delete')) === 'Y')
			{
				if (!$this->getState('j2store_product_id', false))
				{
					$this->log->addStats('incorrect', 'COM_CSVI_NO_PRODUCT_FOUND_TO_DELETE');

					return false;
				}

				$this->deleteProduct();
			}
			else
			{
				// Data must be in an array
				$productData = ArrayHelper::fromObject($this->state);

				$this->productTable->bind($productData);
				$this->productTable->check();

				try
				{
					$this->productTable->save($productData);
					$this->log->add('Product added successfully', false);
					$this->setState('product_id', $this->productTable->j2store_product_id);
					$this->storeProductVariant();
					$this->helper->storeProductQuantity($this->getState('variant_id', false), $this->getState('quantity', 0));
					$this->storeProductImages();

					// Process option_name, option_value and option_type fields
					$this->processVariantsOptionsAndValues();

					// Check if the field is a custom field used as an available field
					$this->processCustomAvailableFields();

					$productId = ($this->getState('product_id', false)) ? $this->getState('product_id', false) : $this->getState('j2store_product_id', false);

					// Finally save the variants
					if ($this->getState('product_type') === 'variable')
					{
						$this->generateVariants($productId);
					}
				}
				catch (\Exception $e)
				{
					$this->log->add('Cannot add product. Error: ' . $e->getMessage(), false);
					$this->log->addStats('incorrect', $e->getMessage());

					return false;
				}
			}
		}

		return true;
	}

	/**
	 * Set product params
	 *
	 * @return  void.
	 *
	 * @since   7.3.0
	 */
	private function setParams()
	{
		if (!$this->getState('params'))
		{
			$paramsFields = array('product_css_class');

			if ($this->getState('product_type', 'simple') === 'downloadable')
			{
				$paramsFields = array('product_css_class', 'download_limit', 'download_expiry');
			}

			$params = json_decode($this->productTable->params);

			if (!is_object($params))
			{
				$params = new \stdClass;
			}

			foreach ($paramsFields as $field)
			{
				$params->$field = $this->getState($field, '');

				if (!$this->getState($field, false))
				{
					$params->$field = '';
				}
			}

			$this->setState('params', json_encode($params));
			$this->productTable->params = json_encode($params);
		}
	}

	/**
	 * Delete product
	 *
	 * @return  void.
	 *
	 * @since   7.3.0
	 */
	private function deleteProduct()
	{
		$variantId = $this->helper->getVariantId($this->getState('sku', false));
		$this->productTable->delete($this->getState('j2store_product_id', false));
		$this->contentTable->delete($this->getState('product_source_id', false));
		$this->variantTable->delete($variantId);

		$query = $this->db->getQuery(true)
			->delete($this->db->quoteName('#__j2store_productimages'))
			->where($this->db->quoteName('product_id') . '=' . (int) $this->getState('j2store_product_id', false));
		$this->db->setQuery($query)->execute();
		$this->log->add('Deleted product images');
		$this->log->addStats('delete', 'COM_CSVI_TABLE_J2STORETABLEPRODUCTIMAGE_DELETED');

		$query->clear()
			->delete($this->db->quoteName('#__j2store_productquantities'))
			->where($this->db->quoteName('variant_id') . '=' . (int) $variantId);
		$this->db->setQuery($query)->execute();
		$this->log->add('Deleted product quantity', false);
		$this->log->addStats('delete', 'COM_CSVI_PRODUCT_QUANTITY_DELETE');

		$query->clear()
			->delete($this->db->quoteName('#__j2store_productfiles'))
			->where($this->db->quoteName('product_id') . '=' . (int) $this->getState('j2store_product_id', false));
		$this->db->setQuery($query)->execute();
		$this->log->add('Deleted product files', false);
		$this->log->addStats('delete', 'COM_CSVI_PRODUCT_FILES_DELETE');

		$query->clear()
			->delete($this->db->quoteName('#__j2store_product_filters'))
			->where($this->db->quoteName('product_id') . '=' . (int) $this->getState('j2store_product_id', false));
		$this->db->setQuery($query)->execute();
		$this->log->add('Deleted product filters', false);
		$this->log->addStats('delete', 'COM_CSVI_PRODUCT_FILTERS_DELETE');

		$query->clear()
			->delete($this->db->quoteName('#__j2store_product_options'))
			->where($this->db->quoteName('product_id') . '=' . (int) $this->getState('j2store_product_id', false));
		$this->db->setQuery($query)->execute();
		$this->log->add('Deleted product options', false);
		$this->log->addStats('delete', 'COM_CSVI_PRODUCT_OPTIONS_DELETE');

		$query->clear()
			->delete($this->db->quoteName('#__j2store_productprice_index'))
			->where($this->db->quoteName('product_id') . '=' . (int) $this->getState('j2store_product_id', false));
		$this->db->setQuery($query)->execute();
		$this->log->add('Deleted product price index', false);
		$this->log->addStats('delete', 'COM_CSVI_PRODUCT_PRICEINDEX_DELETE');

		$query->clear()
			->delete($this->db->quoteName('#__j2store_product_variant_optionvalues'))
			->where($this->db->quoteName('variant_id') . '=' . (int) $variantId);
		$this->db->setQuery($query)->execute();
		$this->log->add('Deleted product variant option values', false);
		$this->log->addStats('delete', 'COM_CSVI_PRODUCT_VARIANT_OPTION_VALUES_DELETE');
	}

	/**
	 * Store product as a Joomla article
	 *
	 * @return  Mixed False if things go wrong.
	 *
	 * @since   7.3.0
	 */
	private function storeJoomlaContent()
	{
		$this->contentTable->load($this->getState('id', false));

		$isNew = false;

		if ($this->contentTable->get('id', false) === false)
		{
			$isNew = true;
		}

		if ($isNew)
		{
			if (!$this->getState('catid', false))
			{
				$this->contentTable->catid = 2;
			}

			// Check if user has enabled field else set to default as published
			$enabled = $this->getState('enabled', false);

			if ($enabled !== false)
			{
				$this->contentTable->state = $enabled;
			}

			if (!$this->getState('language', false))
			{
				$this->contentTable->language = '*';
			}

			if (!$this->getState('access', false))
			{
				$this->contentTable->access = 1;
			}

			if (!$this->getState('modified_on', false))
			{
				$this->contentTable->modified = $this->date->toSql();
			}

			if (!$this->getState('modified_by'))
			{
				$this->contentTable->modified_by = $this->userId;
			}

			if (!$this->getState('j2store_product_id', false))
			{
				if (!$this->getState('created_by'))
				{
					$this->contentTable->created_by = $this->userId;
				}

				if (!$this->getState('created_on'))
				{
					$this->contentTable->created = $this->date->toSql();
				}
			}
		}

		// Data must be in an array
		$data = ArrayHelper::fromObject($this->state);
		$this->contentTable->bind($data);

		// If we dont need have needed fields then we cannot update Joomla content
		if (!$this->contentTable->check())
		{
			return false;
		}

		try
		{
			$this->contentTable->store();
			$this->processTags($this->contentTable->id);
			$this->setState('product_source_id', $this->contentTable->id);

			if ($this->template->get('products_existing_content', false) && !$this->contentTable->checkAlias($this->getState('alias', false)))
			{
				$this->log->add('Product added and linked to existing Joomla content', false);
			}
			else
			{
				$this->log->add('Added product as a Joomla article', false);
			}
		}
		catch (\Exception $e)
		{
			$this->log->add('Cannot add product as Joomla content. Error: ' . $e->getMessage(), false);
			$this->log->addStats('incorrect', $e->getMessage());
		}
	}

	/**
	 * Store product as a variant
	 *
	 * @return  void.
	 *
	 * @since   7.3.0
	 */
	private function storeProductVariant()
	{
		$this->variantTable->load($this->getState('j2store_variant_id', false));

		if (!$this->getState('variant_id', false))
		{
			if (!$this->getState('created_by'))
			{
				$this->variantTable->created_by = $this->userId;
			}

			if (!$this->getState('created_on'))
			{
				$this->variantTable->created_on = $this->date->toSql();
			}

			if (!$this->getState('upc'))
			{
				$this->variantTable->upc = '';
			}

			if ($this->getState('params'))
			{
				$this->variantTable->params = $this->getState('params');
			}

			$arrayDefaultFields = array('length', 'width', 'height', 'length_class_id', 'weight', 'weight_class_id', 'manage_stock', 'min_sale_qty',
				'use_store_config_min_sale_qty', 'max_sale_qty', 'use_store_config_max_sale_qty', 'notify_qty', 'use_store_config_notify_qty',
				'availability');

			foreach ($arrayDefaultFields as $defaultFields)
			{
				$this->variantTable->$defaultFields = 0;

				if ($this->getState($defaultFields, false))
				{
					$this->variantTable->$defaultFields = $this->getState($defaultFields, false);
				}
			}
		}
		else
		{
			if (!$this->getState('modified_on', false))
			{
				$this->variantTable->modified_on = $this->date->toSql();
			}

			if (!$this->getState('modified_by'))
			{
				$this->variantTable->modified_by = $this->userId;
			}
		}

		if (!$this->getState('pricing_calculator', false))
		{
			$this->variantTable->pricing_calculator = 'standard';
		}

		// Set the default value
		$this->variantTable->is_master = 1;

		$variantData = ArrayHelper::fromObject($this->state);
		$this->variantTable->bind($variantData);
		$this->variantTable->check();

		try
		{
			$this->variantTable->save($variantData);
			$this->log->add('Product variant processed', false);
			$this->setState('variant_id', $this->variantTable->j2store_variant_id);
		}
		catch (\Exception $e)
		{
			$this->log->add('Cannot add product as variant. Error: ' . $e->getMessage(), false);
			$this->log->addStats('incorrect', $e->getMessage());
		}
	}

	/**
	 * Process media files.
	 *
	 * @return  bool Returns true on OK | False on failure.
	 *
	 * @since   7.3.0
	 *
	 * @throws  \Exception
	 */
	private function storeProductImages()
	{
		$mainImage              = $this->getState('main_image', false);
		$generateImage          = $this->template->get('auto_generate_image_name', false);
		$productFullImageOutput = '';
		$fileDetails            = array();

		if ($mainImage || $generateImage)
		{
			if (('' === $mainImage) || $generateImage)
			{
				$productFullImageOutput = $this->createImageName();
			}

			// Image handling
			$this->imageHelper = new \CsviHelperImage($this->template, $this->log, $this->csvihelper);
			$thumbImage        = $this->getState('thumb_image', false);
			$productId         = $this->getState('product_id', false);
			$max_width         = $this->template->get('resize_max_width', 1024);
			$max_height        = $this->template->get('resize_max_height', 768);
			$imgPath           = $this->template->get('file_location_product_files', false);

			// Delete existing product images to do a fresh insert
			if ($this->template->get('delete_product_images', false))
			{
				$query = $this->db->getQuery(true)
					->delete($this->db->quoteName('#__j2store_productimages'))
					->where($this->db->quoteName('product_id') . '=' . $productId);
				$this->db->setQuery($query)->execute();
				$this->log->add('Delete product images');
			}

			if ($mainImage)
			{
				$media = [
					'product_id'        => $productId,
					'additional_images' => '[""]'
				];
				$imagePaths           = $this->getImageOriginalFullPath($mainImage);
				$original             = $imagePaths['original'];
				$fullPath             = $imagePaths['fullpath'];
				$media['main_image']  = $mainImage;
				$media['thumb_image'] = empty($thumbImage) ? $media['main_image'] : $thumbImage;

				if ($this->template->get('process_image', false))
				{
					if ($generateImage)
					{
						$fileDetails = $this->imageHelper->processImage($original, $fullPath, $productFullImageOutput);
					}
					else
					{
						$fileDetails = $this->imageHelper->processImage($original, $fullPath);
					}

					$fileDetails['exists']      = true;
					$fileDetails['isimage']     = $this->imageHelper->isImage(JPATH_SITE . '/' . $mainImage);
					$fileDetails['name']        = $mainImage;
					$fileDetails['output_name'] = basename($mainImage);

					if (file_exists(JPATH_SITE . '/' . $mainImage))
					{
						$fileDetails['mime_type'] = $this->imageHelper->findMimeType($mainImage);
					}
					else
					{
						$fileDetails['mime_type'] = '';
					}

					$fileDetails['output_path'] = $fullPath;

					if ($fileDetails['exists'])
					{
						$media['main_image']  = (empty($fileDetails['output_path'])) ? $fileDetails['output_name'] : $fileDetails['output_path'] . $fileDetails['output_name'];
						$media['thumb_image'] = '';

						if (($fileDetails['isimage']) && $this->template->get('thumb_create', false))
						{
							if ($thumbImage)
							{
								// Check if the image contains the image path
								$dirname = dirname($thumbImage);

								if (strpos($imgPath . 'thumbs/', $dirname) !== false)
								{
									// Collect rest of folder path if it is more than image default path
									$imageLeftPath = str_replace($imgPath, '', $dirname . '/');
									$imageThumb    = basename($thumbImage);

									if ($imageLeftPath)
									{
										$thumbImage = $imageLeftPath . $imageThumb;
									}
								}
							}

							$thumbSizes = getimagesize(JPATH_SITE . '/' . $media['main_image']);

							if (empty($thumbImage) || $generateImage)
							{
								// Get the subfolder structure
								$thumbPath  = str_ireplace($imgPath, '', $fullPath);
								$thumbImage = $thumbPath . 'thumbs/' . basename($media['main_image']);
							}
							else
							{
								// Check if we are not overwriting any large images
								$thumbPathParts = pathinfo($thumbImage);

								if ($thumbPathParts['dirname'] === '.')
								{
									$this->log->addStats('incorrect', 'COM_CSVI_THUMB_OVERWRITE_FULL');
									$thumbImage = false;
								}
							}

							if ($thumbImage && ($thumbSizes[0] < $max_width || $thumbSizes[1] < $max_height))
							{
								$media['thumb_image'] = $this->imageHelper->createThumbnail($media['main_image'], $imgPath, $thumbImage);
							}
							else
							{
								$this->log->addStats('incorrect', Text::sprintf('COM_CSVI_THUMB_TOO_BIG', $max_width, $max_height, $thumbSizes[0], $thumbSizes[1]));
								$this->log->add('Thumbnail is bigger than maximums set', false);
								$media['thumb_image'] = '';
							}
						}
						else
						{
							$media['thumb_image'] = empty($thumbImage) ? $media['main_image'] : $thumbImage;

							if (0 === strpos($media['thumb_image'], 'http') && null !== $thumbImage)
							{
								$media['thumb_image'] = $thumbImage;
							}
						}
					}
					else
					{
						$media['thumb_image'] = '';
					}
				}

				// Check for additional images
				$additionalImages          = explode('|', $this->getState('additional_images', false));
				$additionalImagesNew       = new \stdClass;
				$additionalFullImageOutput = '';

				if ($this->getState('additional_images', false))
				{
					foreach ($additionalImages as $key => $additionalImage)
					{
						$additionalImage = trim($additionalImage);

						if (count($additionalImages) === 1)
						{
							$imgCounter = 1;
						}
						else
						{
							$imgCounter = $key + 1;
						}

						if ($generateImage)
						{
							$additionalFullImageOutput = $this->createImageName($imgCounter);
						}

						$additionalImagePaths = $this->getImageOriginalFullPath($additionalImage);
						$additionalOriginal   = $additionalImagePaths['original'];
						$additionalFullPath   = $additionalImagePaths['fullpath'];

						if ($this->template->get('process_image', false))
						{
							if ($generateImage)
							{
								$additionalFileDetails = $this->imageHelper->processImage($additionalOriginal, $additionalFullPath, $additionalFullImageOutput);
							}
							else
							{
								$additionalFileDetails = $this->imageHelper->processImage($additionalOriginal, $additionalFullPath);
							}

							$additionalFileDetails['exists']      = true;
							$additionalFileDetails['isimage']     = $this->imageHelper->isImage(JPATH_SITE . '/' . $additionalImage);
							$additionalFileDetails['name']        = $additionalImage;
							$additionalFileDetails['output_name'] = basename($additionalImage);

							if (file_exists(JPATH_SITE . '/' . $additionalImage))
							{
								$additionalFileDetails['mime_type'] = $this->imageHelper->findMimeType($additionalImage);
							}
							else
							{
								$additionalFileDetails['mime_type'] = '';
							}

							$additionalFileDetails['output_path'] = $additionalFullPath;

							if ($additionalFileDetails['exists'])
							{
								if ($this->template->get('full_watermark', 'image') && $fileDetails['isimage'])
								{
									$this->imageHelper->addWatermark(JPATH_SITE . '/' . $additionalImage);
								}

								$additionalImagesNew->$key = (empty($additionalFileDetails['output_path'])) ? $additionalFileDetails['output_name'] : $additionalFileDetails['output_path'] . $additionalFileDetails['output_name'];
							}
						}
					}

					$media['additional_images'] = json_encode($additionalImagesNew);
				}

				$this->imageTable->load(['product_id' => $productId]);

				// Bind the media data
				$this->imageTable->bind($media);

				try
				{
					$this->imageTable->store();

					$this->log->add('Image added for the product', false);

					if ($this->template->get('full_watermark', 'image') && $fileDetails['isimage'])
					{
						$this->imageHelper->addWatermark(JPATH_SITE . '/' . $media['main_image']);
					}
				}
				catch (\Exception $e)
				{
					$this->log->add('Cannot add product image. Error: ' . $e->getMessage(), false);
					$this->log->addStats('incorrect', $e->getMessage());

					return false;
				}

				$this->imageTable->reset();
			}
		}

		return true;
	}

	/**
	 * Create original and fullpath of image
	 *
	 * Check if the user wants to have CSVI J2Store create the image names if so
	 * create the image names without path.
	 *
	 * @param   int  $imageName  The name of the image
	 *
	 * @return  array  The path of the image.
	 *
	 * @since   7.3.0
	 */
	private function getImageOriginalFullPath($imageName)
	{
		$image   = array();
		$imgPath = $this->template->get('file_location_product_files', false);
		$imgPath = str_replace('\\', '/', $imgPath);

		if (substr($imgPath, -1) !== '/')
		{
			$imgPath .= '/';
		}

		if ($this->imageHelper->isRemote($imageName))
		{
			$image['original'] = $imageName;
			$image['fullpath'] = $imgPath;
		}
		else
		{
			$imageName = str_replace('\\', '/', $imageName);

			// Check if the image contains the image path
			$dirname = dirname($imageName) . '/';

			if (strpos($dirname, $imgPath) !== false && $imgPath !== '/')
			{
				// Collect rest of folder path if it is more than image default path
				$imageLeftPath = str_replace($imgPath, '', $dirname . '/');
				$imageName     = basename($imageName);

				if ($imageLeftPath)
				{
					$imageName = $imageLeftPath . $imageName;
				}
			}

			$image['original'] = $imgPath . $imageName;

			// Get subfolders
			$pathParts         = pathinfo($image['original']);
			$image['fullpath'] = $pathParts['dirname'] . '/';
		}

		return $image;
	}

	/**
	 * Create image name.
	 *
	 * Check if the user wants to have CSVI J2Store create the image names if so
	 * create the image names without path.
	 *
	 * @param   int  $ordering  The number to apply to a generated image name.
	 *
	 * @return  string  The name of the image.
	 *
	 * @since   7.3.0
	 */
	private function createImageName($ordering = 0)
	{
		$this->log->add('Generate image name', false);

		// Create extension
		$ext = $this->template->get('autogenerateext');

		// Check if the user wants to convert the images to a different type
		switch ($this->template->get('type_generate_image_name'))
		{
			case 'product_sku':
				$this->log->add('Create name from product SKU', false);

				if (!$this->getState('sku', false))
				{
					$this->log->addStats('error', 'COM_CSVI_CANNOT_FIND_PRODUCT_SKU');

					return false;
				}

				$name = $this->getState('sku');
				break;
			case 'product_name':
				$this->log->add('Create name from product name', false);

				if (!$this->getState('title', false))
				{
					$this->log->addStats('error', 'COM_CSVI_CANNOT_FIND_PRODUCT_NAME');

					return false;
				}

				$name = str_replace(' ', '_', $this->getState('title', false));
				break;
			case 'product_id':
				$this->log->add('Create name from product ID', false);

				if (!$this->getState('j2store_product_id'))
				{
					$this->log->addStats('error', 'COM_CSVI_CANNOT_FIND_PRODUCT_ID');

					return false;
				}

				$name = $this->getState('j2store_product_id');
				break;
			case 'random':
				$this->log->add('Create a random name', false);
				$name = mt_rand();
				break;
			default:
				$this->log->addStats('error', 'COM_CSVI_CANNOT_FIND_PRODUCT_SKU');

				return false;
				break;
		}

		// Build the new name
		$imageName = $name . '.' . $ext;

		if ($ordering > 0)
		{
			$imageName = $name . '_' . $ordering . '.' . $ext;
		}

		$this->log->add('Created image name: ' . $imageName, false);

		// Check if the user is supplying image data
		if (!$this->getState('main_image', false))
		{
			$this->setState('main_image', $imageName);
		}

		return $imageName;
	}

	/**
	 * Load the necessary tables.
	 *
	 * @return  void.
	 *
	 * @since   7.3.0
	 */
	public function loadTables()
	{
		$this->productTable = $this->getTable('Product');
		$this->contentTable = $this->getTable('Content');
		$this->variantTable = $this->getTable('Variant');
		$this->imageTable   = $this->getTable('ProductImage');
	}

	/**
	 * Clear the loaded tables.
	 *
	 * @return  void.
	 *
	 * @since   7.3.0
	 */
	public function clearTables()
	{
		$this->productTable->reset();
		$this->contentTable->reset();
		$this->variantTable->reset();
		$this->imageTable->reset();
	}

	/**
	 * Process product options and values
	 *
	 * @return  mixed false if needed fields are not set.
	 *
	 * @since   7.7.0
	 */
	private function processVariantsOptionsAndValues()
	{
		if (!$this->getState('option_names', false) || !$this->getState('option_values', false))
		{
			return false;
		}

		$options      = explode('~', $this->getState('option_names', false));
		$optionValues = explode('~', $this->getState('option_values', false));

		if (count($options) === count($optionValues))
		{
			$excludeFieldTypes = ['text', 'textarea', 'file', 'date', 'time', 'datetime'];

			$query = $this->db->getQuery(true)
				->select($this->db->quoteName('type'))
				->from($this->db->quoteName('#__j2store_options'));

			foreach ($options as $key => $name)
			{
				$query->clear('where')
					->where($this->db->quoteName('option_unique_name') . ' = ' . $this->db->quote($name));
				$this->db->setQuery($query);
				$fieldType = $this->db->loadResult();

				if (in_array($fieldType, $excludeFieldTypes))
				{
					$this->log->add('Field ' . $name . ' has type ' . $fieldType . ' and is not processed as this has no options in product configuration.');
					$this->log->addStats('information', Text::sprintf('COM_CSVI_IGNORE_FIELDTYPE', $fieldType));

					continue;
				}

				$this->storeCustomFields($name, $optionValues[$key]);
			}
		}
		else
		{
			$this->log->add('Option name and Option values are not of same count');
			$this->log->addStats('error', 'COM_CSVI_OPTION_COUNT_NOT_SAME');
		}
	}

	/**
	 * Method to store custom fields
	 *
	 * @param  string  $fieldName  Name of the custom field
	 * @param  string  $fieldValue Value of the custom field
	 *
	 * @return  bool True if all Ok False otherwise.
	 *
	 * @since   7.15.0
	 */
	private function storeCustomFields($fieldName, $fieldValue)
	{
		$productId = ($this->getState('product_id', false)) ? $this->getState('product_id', false) : $this->getState('j2store_product_id', false);

		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('j2store_option_id'))
			->from($this->db->quoteName('#__j2store_options'))
			->where($this->db->quoteName('option_unique_name') . ' = ' . $this->db->quote($fieldName));

		$this->log->add('Find the option name id');
		$this->db->setQuery($query);
		$optionNameId = $this->db->loadResult();

		if (!$optionNameId)
		{
			$this->log->add('Find the option name id');
			$this->log->addStats('error', Text::sprintf('COM_CSVI_OPTION_NAME_NOT_FOUND', $fieldName));

			return false;
		}

		$optionValueArray = explode('#', $fieldValue);

		foreach ($optionValueArray as $k => $value)
		{
			$query->clear()
				->select($this->db->quoteName('j2store_optionvalue_id'))
				->from($this->db->quoteName('#__j2store_optionvalues'))
				->where($this->db->quoteName('option_id') . ' = ' . (int) $optionNameId)
				->where($this->db->quoteName('optionvalue_name') . ' = ' . $this->db->quote($value));
			$this->log->add('Find the option value id');
			$this->db->setQuery($query);
			$optionValueId = $this->db->loadResult();

			if (!$optionValueId)
			{
				$query->clear()
					->insert($this->db->quoteName('#__j2store_optionvalues'))
					->columns($this->db->quoteName('optionvalue_name') . ', ' . $this->db->quoteName('option_id'))
					->values($this->db->quote($value) . ', ' . (int) $optionNameId);
				$this->log->add('Insert new option name');
				$this->db->setQuery($query)->execute();
				$optionValueId = $this->db->insertid();
			}

			$query->clear()
				->select($this->db->quoteName('j2store_productoption_id'))
				->from($this->db->quoteName('#__j2store_product_options'))
				->where($this->db->quoteName('option_id') . ' = ' . (int) $optionNameId)
				->where($this->db->quoteName('product_id') . ' = ' . (int) $productId);
			$this->log->add('Find the option value id');
			$this->db->setQuery($query);
			$productOptionNameId = $this->db->loadResult();

			if (!$productOptionNameId)
			{
				// Insert the product options first
				$query->clear()
					->insert($this->db->quoteName('#__j2store_product_options'))
					->columns($this->db->quoteName('option_id') . ', ' . $this->db->quoteName('product_id'))
					->values((int) $optionNameId . ', ' . (int) $productId);
				$this->log->add('Insert new product option name');
				$this->db->setQuery($query)->execute();
				$productOptionNameId = $this->db->insertid();

			}

			$query->clear()
				->select($this->db->quoteName('j2store_product_optionvalue_id'))
				->from($this->db->quoteName('#__j2store_product_optionvalues'))
				->where($this->db->quoteName('productoption_id') . ' = ' . (int) $productOptionNameId)
				->where($this->db->quoteName('optionvalue_id') . ' = ' . (int) $optionValueId);
			$this->log->add('Find the product option value id');
			$this->db->setQuery($query);
			$productOptionValueId = $this->db->loadResult();

			if (!$productOptionValueId)
			{
				$query->clear()
					->insert($this->db->quoteName('#__j2store_product_optionvalues'))
					->columns($this->db->quoteName('productoption_id') . ', ' . $this->db->quoteName('optionvalue_id'))
					->values((int) $productOptionNameId . ', ' . (int) $optionValueId);
				$this->log->add('Insert new product option value');
				$this->db->setQuery($query)->execute();
			}
		}

		return true;
	}

	/**
	 * Generate Variants and save them
	 *
	 * @param int $productId Product Id
	 *
	 * @return  bool True if all Ok False otherwise.
	 *
	 * @since   7.7.0
	 */
	private function generateVariants($productId)
	{
		// Collect all the existing variant Ids for the product
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('j2store_variant_id'))
			->from($this->db->quoteName('#__j2store_variants'))
			->where($this->db->quoteName('product_id') . ' = ' . (int) $productId)
			->where($this->db->quoteName('is_master') . ' = 0');
		$this->db->setQuery($query);
		$variantIds         = $this->db->loadRowList();
		$existingVariantIds = array();

		foreach ($variantIds as $vIds)
		{
			$existingVariantIds[] = $vIds[0];
		}

		// Delete variants in database if user wants to
		if ($this->template->get('delete_variants', false))
		{
			$query = $this->db->getQuery(true)
				->delete($this->db->quoteName('#__j2store_variants'))
				->where($this->db->quoteName('product_id') . ' = ' . (int) $productId)
				->where($this->db->quoteName('is_master') . ' = 0');
			$this->log->add('Delete the product variant entries before inserting');
			$this->db->setQuery($query)->execute();

			if ($existingVariantIds)
			{
				$query->clear()
					->delete($this->db->quoteName('#__j2store_product_variant_optionvalues'))
					->where($this->db->quoteName('variant_id') . ' IN  (' . implode(',', $existingVariantIds) . ')');
				$this->db->setQuery($query)->execute();
				$this->log->add('Delete variant and product option combinations');
			}
		}

		// Generate variants with combinations
		if ($this->template->get('generate_variants', false))
		{
			$query = $this->db->getQuery(true)
				->select($this->db->quoteName('j2store_productoption_id'))
				->from($this->db->quoteName('#__j2store_product_options'))
				->where($this->db->quoteName('product_id') . ' = ' . (int) $productId);
			$this->log->add('Find the option value id');
			$this->db->setQuery($query);
			$productOptions = $this->db->loadObjectList();

			$valueIds = array();

			foreach ($productOptions as $productoption)
			{
				$query->clear()
					->select($this->db->quoteName('j2store_product_optionvalue_id'))
					->from($this->db->quoteName('#__j2store_product_optionvalues'))
					->where($this->db->quoteName('productoption_id') . ' = ' . (int) $productoption->j2store_productoption_id);
				$this->db->setQuery($query);
				$optionValues = $this->db->loadObjectList();

				if ($optionValues)
				{
					$optionIds = array();

					foreach ($optionValues as $optValues)
					{
						$optionIds[] = $optValues->j2store_product_optionvalue_id;
					}

					$valueIds[] = $optionIds;
				}
			}

			if (!$valueIds)
			{
				return false;
			}

			$combinations      = $this->getCombinations($valueIds);
			$combinationResult = array();

			foreach ($combinations as $combination)
			{
				$values = explode(',', $combination);
				sort($values);
				$combinationResult[] = implode(',', $values);
			}

			foreach ($combinationResult as $combResult)
			{
				$existingId = '';

				if ($existingVariantIds)
				{
					$query->clear()
						->select($this->db->quoteName('variant_id'))
						->from($this->db->quoteName('#__j2store_product_variant_optionvalues'))
						->where($this->db->quoteName('variant_id') . ' IN  (' . implode(',', $existingVariantIds) . ')')
						->where($this->db->quoteName('product_optionvalue_ids') . ' = ' . $this->db->quote($combResult));
					$this->db->setQuery($query);
					$existingId = $this->db->loadResult();
					$this->log->add('Check if variant combination already exists');
				}

				if (!$existingId)
				{
					$query = $this->db->getQuery(true)
						->insert($this->db->quoteName('#__j2store_variants'))
						->columns($this->db->quoteName('product_id') . ', ' . $this->db->quoteName('pricing_calculator')
							. ', ' . $this->db->quoteName('is_master') . ', ' . $this->db->quoteName('shipping'))
						->values((int) $productId . ', ' . $this->db->quote('standard') . ', 0, ' . $this->getState('shipping', 0));
					$this->log->add('Insert new variant');
					$this->db->setQuery($query)->execute();
					$variantId = $this->db->insertid();

					$sku = 'Variable_' . $variantId;

					$query->clear()
						->update($this->db->quoteName('#__j2store_variants'))
						->set($this->db->quoteName('sku') . ' = ' . $this->db->quote($sku))
						->where($this->db->quoteName('j2store_variant_id') . ' = ' . (int) $variantId);
					$this->log->add('Update the SKU for new variant');
					$this->db->setQuery($query)->execute();

					if ($variantId)
					{
						$query->clear()
							->delete($this->db->quoteName('#__j2store_product_variant_optionvalues'))
							->where($this->db->quoteName('variant_id') . ' = ' . (int) $variantId);
						$this->log->add('Delete the variant entries before inserting');
						$this->db->setQuery($query)->execute();

						$query->clear()
							->insert($this->db->quoteName('#__j2store_product_variant_optionvalues'))
							->columns($this->db->quoteName('variant_id') . ', ' . $this->db->quoteName('product_optionvalue_ids'))
							->values((int) $variantId . ', ' . $this->db->quote($combResult));
						$this->log->add('Insert new variant');
						$this->db->setQuery($query)->execute();
					}
				}
				else
				{
					$query->clear()
						->update($this->db->quoteName('#__j2store_variants'))
						->set($this->db->quoteName('shipping') . ' = ' . (int) $this->getState('shipping', 0))
						->set($this->db->quoteName('product_id') . ' = ' . (int) $productId)
						->where($this->db->quoteName('j2store_variant_id') . ' = ' . (int) $existingId);
					$this->log->add('Update variant');
					$this->db->setQuery($query)->execute();
				}
			}
		}

		return true;
	}

	/**
	 * Get the combination of variants
	 *
	 * @param array $valueIds Array of option value ids
	 *
	 * @return  array Combination of variants.
	 *
	 * @since   7.7.0
	 */
	private function getCombinations($valueIds)
	{
		$result         = array();
		$maxCombination = 1;

		foreach ($valueIds as $valueId)
		{
			$maxCombination = $maxCombination * count($valueId);
		}

		for ($i = 0; $i < $maxCombination; $i++)
		{
			$output   = '';
			$quotient = $i;

			foreach (array_reverse($valueIds) as $valueId)
			{
				$divisor   = count($valueId);
				$remainder = $quotient % $divisor;
				$quotient  = $quotient / $divisor;
				$output    = $valueId[$remainder] . ',' . $output;
			}

			$result[] = trim($output, ',');
		}

		return $result;
	}

	/**
	 * Get a list of custom fields that can be used as available field.
	 *
	 * @return  void.
	 *
	 * @since   7.15.0
	 *
	 * @throws  \RuntimeException
	 */
	private function loadCustomFields()
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('option_unique_name'))
			->from($this->db->quoteName('#__j2store_options'))
			->where($this->db->quoteName('enabled') . ' = 1');
		$this->db->setQuery($query);
		$result = $this->db->loadColumn();

		if (!is_array($result))
		{
			$result = array();
		}

		$this->customFields = $result;
	}

	/**
	 * Process custom fields that are used as available field.
	 *
	 * @return  void.
	 *
	 * @since   7.15.0
	 *
	 */
	private function processCustomAvailableFields()
	{
		if (count($this->customFields) > 0)
		{
			foreach ($this->customFields as $field)
			{
				if ($this->getState($field, false))
				{
					$this->storeCustomFields($field, $this->getState($field, false));
				}
			}
		}
	}

	/**
	 * Initialise Categories import file
	 *
	 * @return  void.
	 *
	 * @since   7.15.0
	 *
	 */
	private function initialiseCategoriesImport()
	{
		$this->categoryImportModel = $this->fields->initialiseAnotherImport('categories', 'com_categories', 'category');
	}

	/**
	 * Process custom fields that are used as available field.
	 *
	 * @param   string  $categoryPath  Category path
	 *
	 * @return  int category id.
	 *
	 * @since   7.15.0
	 *
	 */
	private function runCategoryImport($categoryPath)
	{
		if ($categoryPath)
		{
			$this->template->set('category_separator', '/');

			if (strpos($categoryPath, '|') !== false)
			{
				$splitCategory = explode('|', $categoryPath);
				$categoryPath  = array_shift($splitCategory);
			}

			$this->categoryImportModel->setLoaded();
			$this->categoryImportModel->loadTables();
			$this->categoryImportModel->setState('path', $categoryPath);
			$this->categoryImportModel->setState('extension', 'com_content');
			$this->categoryImportModel->setState('id', 0);
			$this->categoryImportModel->getProcessRecord();
			$this->categoryImportModel->clearTables();
			$categoryId = $this->categoryImportModel->getState('id');

			return $categoryId;
		}
	}

	/**
	 * Update Tags data
	 *
	 * @param   int  $id  Id of the article
	 *
	 * @return  bool Returns true if all is OK | Returns false otherwise
	 *
	 * @since   7.18.0
	 */
	private function processTags($id)
	{
		$tags = $this->getState('tags', false);

		if (!$tags)
		{
			return false;
		}

		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('core_content_id'))
			->from($this->db->quoteName('#__ucm_content'))
			->where($this->db->quoteName('core_alias') . '  = ' . $this->db->quote($this->getState('alias')));
		$this->db->setQuery($query);
		$ucmId = $this->db->loadResult();

		if ($ucmId)
		{
			$query->clear()
				->delete($this->db->quoteName('#__ucm_content'))
				->where($this->db->quoteName('core_content_id') . ' = ' . (int) $ucmId);
			$this->db->setQuery($query)->execute();

			$query->clear()
				->delete($this->db->quoteName('#__ucm_base'))
				->where($this->db->quoteName('ucm_id') . ' = ' . (int) $ucmId);
			$this->db->setQuery($query)->execute();
		}

		$content = $this->getState('fulltext');

		if (!$content && $this->getState('introtext'))
		{
			$content = $this->getState('introtext');
		}

		$query->clear()
			->insert($this->db->quoteName('#__ucm_content'))
			->columns($this->db->quoteName(array('core_type_alias', 'core_title', 'core_alias', 'core_body', 'core_state', 'core_access', 'core_content_item_id', 'core_type_id', 'core_catid', 'core_params', 'core_metadata', 'core_images', 'core_urls')))
			->values($this->db->quote('com_content.article') . ', ' . $this->db->quote($this->getState('title')) . ', ' . $this->db->quote($this->getState('alias')) . ', ' . $this->db->quote($content) . ', 1, 1, ' .
				(int) $this->getState('id') . ', 1, ' . (int) $this->getState('catid') . ',' . $this->db->quote($this->getState('attribs')) . ',' . $this->db->quote($this->getState('metadata')) . ',' . $this->db->quote($this->getState('images')) . ',' . $this->db->quote($this->getState('urls')));
		$this->db->setQuery($query)->execute();
		$ucmId = $this->db->insertid();

		$tagsArray = explode('|', $tags);

		$typeAlias = 'com_content.article';
		$query->clear()
			->select($this->db->quoteName('type_id'))
			->from($this->db->quoteName('#__content_types'))
			->where($this->db->quoteName('type_alias') . '  = ' . $this->db->quote($typeAlias));
		$this->db->setQuery($query);
		$typeId  = $this->db->loadResult();
		$tagDate = $this->date->toSql();

		foreach ($tagsArray as $tag)
		{
			$query->clear()
				->select($this->db->quoteName('id'))
				->from($this->db->quoteName('#__tags'))
				->where($this->db->quoteName('path') . '  = ' . $this->db->quote($tag));
			$this->db->setQuery($query);
			$tagId = $this->db->loadResult();
			$this->log->add('Get the tag id ');

			if (!$tagId)
			{
				$this->log->add('No tag id found for the tag ' . $tag, false);
				continue;
			}

			// Delete the values and do a fresh import to avoid dulicate error
			$query->clear()
				->delete($this->db->quoteName('#__contentitem_tag_map'))
				->where($this->db->quoteName('content_item_id') . ' = ' . (int) $id)
				->where($this->db->quoteName('tag_id') . ' = ' . (int) $tagId);
			$this->db->setQuery($query)->execute();
			$this->log->add('Removed existing tag for content before inserting');

			$query->clear()
				->insert($this->db->quoteName('#__contentitem_tag_map'))
				->columns($this->db->quoteName(array('type_alias', 'content_item_id', 'tag_id', 'tag_date', 'type_id', 'core_content_id',)))
				->values($this->db->quote($typeAlias) . ', ' . (int) $id . ', ' .
					(int) $tagId . ', ' . $this->db->quote($tagDate) . ', ' . (int) $typeId . ', ' . (int) $ucmId);
			$this->db->setQuery($query)->execute();
			$this->log->add('Insert the new tag for content');
		}

		// Do insert to #__ucm_base table
		$query->clear()
			->insert($this->db->quoteName('#__ucm_base'))
			->columns($this->db->quoteName(array('ucm_id', 'ucm_item_id', 'ucm_type_id')))
			->values((int) $ucmId . ',' . (int) $id . ', ' . (int) $typeId );
		$this->db->setQuery($query)->execute();

		return true;
	}
}
